<?php

namespace Drupal\mediaelement\Form;

use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for MediaElement.js module.
 */
class MediaElementConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'mediaelement_config';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'mediaelement.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('mediaelement.settings');

    // Create global settings config items.
    $global_settings = [];
    $global_settings['attach_sitewide'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable MediaElement.js site wide'),
      '#description' => $this->t('Attach the MediaElement.js library throughtout the entire site. Any <code>audio</code> or <code>video</code> HTML tag will have the player applied to them.'),
      '#default_value' => $config->get('attach_sitewide') ?? FALSE,
      '#weight' => 0,
    ];

    // Create Global Player settings config items.
    $player_settings = [];

    $player_settings['class_prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Class Prefix'),
      '#description' => $this->t('Class prefix for player elements.'),
      '#default_value' => $config->get('class_prefix') ?? '',
      '#placeholder' => 'mejs__',
    ];

    $player_settings['set_dimensions'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Set Dimensions'),
      '#description' => $this->t('Set dimensions via JS instead of CSS.'),
      '#default_value' => $config->get('set_dimensions') ?? TRUE,
      '#weight' => 0,
    ];

    $video_settings = [];

    $video_settings['default_video_width'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Video Width'),
      '#description' => $this->t('Default width if the <code>&#60;video&#62;</code> width is not specified'),
      '#default_value' => $config->get('default_video_width') ?? '',
      '#placeholder' => '480',
    ];

    $video_settings['default_video_height'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Video Height'),
      '#description' => $this->t('Default width if the <code>&#60;video&#62;</code> height is not specified'),
      '#default_value' => $config->get('default_video_height') ?? '',
      '#placeholder' => '270',
    ];

    $video_settings['video_width'] = [
      '#type' => 'number',
      '#title' => $this->t('Video Width'),
      '#description' => $this->t('If set, overrides <code>&#60;video&#62;</code> width'),
      '#default_value' => $config->get('video_width') ?? '',
      '#placeholder' => '-1',
    ];

    $video_settings['video_height'] = [
      '#type' => 'number',
      '#title' => $this->t('Video Height'),
      '#description' => $this->t('If set, overrides <code>&#60;video&#62;</code> height'),
      '#default_value' => $config->get('video_height') ?? '',
      '#placeholder' => '-1',
    ];

    $audio_settings = [];

    $audio_settings['default_audio_width'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Audio Width'),
      '#description' => $this->t('Default width if the <code>&#60;audio&#62;</code> width is not specified'),
      '#default_value' => $config->get('default_audio_width') ?? '',
      '#placeholder' => '400',
    ];

    $audio_settings['default_audio_height'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Audio Height'),
      '#description' => $this->t('Default width if the <code>&#60;audio&#62;</code> height is not specified'),
      '#default_value' => $config->get('default_audio_height') ?? '',
      '#placeholder' => '30',
    ];

    $audio_settings['audio_width'] = [
      '#type' => 'number',
      '#title' => $this->t('Audio Width'),
      '#description' => $this->t('If set, overrides <code>&#60;audio&#62;</code> width'),
      '#default_value' => $config->get('audio_width') ?? '',
      '#placeholder' => '-1',
    ];

    $audio_settings['audio_height'] = [
      '#type' => 'number',
      '#title' => $this->t('Audio Height'),
      '#description' => $this->t('If set, overrides <code>&#60;audio&#62;</code> height'),
      '#default_value' => $config->get('audio_height') ?? '',
      '#placeholder' => '-1',
    ];

    // Attach forms arrays to the form output.
    $form['global_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Global Settings'),
    ];

    $api_link = Link::fromTextAndUrl(
      $this->t('API Documentation'),
      Url::fromUri('https://github.com/mediaelement/mediaelement/blob/master/docs/api.md#mediaelementplayer')
    );

    $form['global_settings']['player_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Player Settings'),
      '#description' => $this->t('For a full explaination of configuration options, see the @api_link.', [
        '@api_link' => $api_link->toString(),
      ]),
      '#weight' => 1,
    ];

    $form['global_settings']['player_settings']['video_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Video Settings'),
      '#weight' => 2,
    ];

    $form['global_settings']['player_settings']['audio_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Audio Settings'),
      '#weight' => 2,
    ];

    $form['global_settings'][] = $global_settings;
    $form['global_settings']['player_settings'][] = $player_settings;
    $form['global_settings']['player_settings']['video_settings'][] = $video_settings;
    $form['global_settings']['player_settings']['audio_settings'][] = $audio_settings;

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('mediaelement.settings');
    $config->set('attach_sitewide', $form_state->getValue('attach_sitewide'));

    $settings = [
      'class_prefix',
      'set_dimensions',
      'default_video_width',
      'default_video_height',
      'default_audio_width',
      'default_audio_height',
      'video_width',
      'video_height',
      'audio_width',
      'audio_height',
    ];

    foreach ($settings as $setting) {
      $value = $form_state->getValue($setting);

      // Only set boolean or passed value. If something is empty, fall back to
      // MediaElement default config.
      if (!empty($value) || $value === 0) {
        $config->set($setting, $value);
      }
      else {
        $config->clear($setting);
      }
    }

    $config->save();
    parent::submitForm($form, $form_state);
  }

}
